#!/usr/bin/perl

#use diagnostics;
use warnings;
use strict;
use Getopt::Std;
use Net::SMTP::SSL;
use Net::SMTPS;
use DBI;
use Config::IniFiles;
use Net::IP qw(:PROC);
use utf8;
use Encode qw(decode encode);

sub format_mail {
	my $txt = $_[0];
	my $dat = $_[1];
	#my $ip = new Net::IP ($dat->{device_device_ip_addr }) or die (Net::IP::Error());
	my $sip = ip_inttobin($dat->{device_ip_addr},$dat->{device_ip_version}); #on converti l'IP en binaire
	$sip = ip_bintoip($sip,$dat->{device_ip_version}); ##on l'a format en xxx.xxx.xxx.xxx
	my $service_lc = lc($dat->{service_name});
    $txt =~ s/\%node.group/$dat->{device_group_name}/gi;
	$txt =~ s/\%node.name/$dat->{device_name}/gi;
	$txt =~ s/\%node.ip/$sip/gi;
	$txt =~ s/\%node.description/$dat->{device_description}/gi;
	$txt =~ s/\%service.name/$service_lc/gi;
	$txt =~ s/\%service.parameter/$dat->{service_parameter}/gi;
	$txt =~ s/\%error.type/$dat->{error_type}/gi;
	$txt =~ s/\%error.name/$dat->{error_name}/gi;
	$txt =~ s/\%error.time/$dat->{happen_timestamp}/gi;
	$txt =~ s/\%error.message/$dat->{happen_message}/gi;
	$txt =~ s/\\n/\n/gi;
	if($dat->{error_type} eq 'critical')
	{
		$txt =~ s/\%color/#DA002E/gi;
	}
	elsif($dat->{error_type} eq 'warning')
	{
		$txt =~ s/\%color/#FFCB1F/gi;
	}
	elsif($dat->{error_type} eq 'unknwon')
	{
		$txt =~ s/\%color/#575759/gi;
	}
        elsif($dat->{error_type} eq 'ok')
        {
                $txt =~ s/\%color/#83BC00/gi;
        }
	else
	{
		$txt =~ s/\%color/#FFFFFF/gi;
	}
	return $txt;
#Noeud : %node.name : %node.ip \n Service: %service   ETAT: %error.type Heure: %error.time \n %error.message
}

my %opts;
my $config="/etc/esia/esia.conf";
#test des paramètres
if(!getopts( "e:t:c:hD", \%opts ))
{
  print "Usage : send_mail.pl -e <id_error> -t <to> -c <esia config file>\n";
  exit(1);
}
 my $debug = 0;
if($opts{'D'}){
  $debug = 1;
}
if($opts{'h'})
{
  print "Usage : send_mail.pl -e <id_error> -t <to> -c <esia config file>\n"; exit(1);
}
if(!$opts{'e'})
{
   print "no error is defined\n"; exit(1);
}
if($opts{'c'})
{
  $config=$opts{'c'};
}
if(!$opts{'t'})
{
  print "mail to not defined\n"; exit(1);
}
#chargement du fichier de configuration
my $cfg = new Config::IniFiles( -file => $config  )or die "esia config file not found (default: /etc/esia/esia.conf)";
#Connexion à la base de donnée
my $db = DBI -> connect($cfg->val( 'POSTGRESQL', 'PGSQL_connect' ),$cfg->val( 'POSTGRESQL', 'PGSQL_username' ),$cfg->val( 'POSTGRESQL', 'PGSQL_pwd' )) or die "Connection DB Failed";
#connexion au serveur SMTP
my $to = $opts{'t'};
my $host;
my $port=25;
my $smtp;
my $user;
my $password;
my $from;
my $message;
my $subject;
my $query;

if( $opts{'e'} eq 'min' )
{
	$query=$db->prepare("SELECT MIN(happen_id) as hi_min from esia_error_happen;");
	$query->execute();
	$opts{'e'} = $query->fetchrow_hashref->{hi_min};
}

$query = $db->prepare("SELECT config_key,config_value
                          FROM esia_configuration
                          WHERE config_key LIKE 'MAIL_%';");
$query->execute();
while (my @data = $query->fetchrow_array ) {
	if($data[0] eq 'MAIL_SMTP')
	{
		$host = $data[1];
	}
	elsif($data[0] eq 'MAIL_SMTP_PORT')
	{
		$port = $data[1];
	}
	elsif($data[0] eq 'MAIL_USER')
	{
		$user = $data[1];
	}
	elsif($data[0] eq 'MAIL_PASSWORD')
	{
		$password = $data[1];
	}
	elsif($data[0] eq 'MAIL_FROM')
	{
		$from = $data[1];
	}
	elsif($data[0] eq 'MAIL_FORMAT_MESSAGE')
	{
		$message = $data[1];
	}
	elsif($data[0] eq 'MAIL_FORMAT_SUBJECT')
	{
		$subject = $data[1];
	}
}

my $hello = $from =~ s/@/\./r;
print"$host:$port user:$user from:$from helo:$hello\n" if $debug;
if($port == '465'){
	print "Secure mode\n" if $debug;
	#connexion sécurisée
	$smtp = Net::SMTP::SSL->new(Host =>$host,Hello=>$hello ,Port=> $port,Timeout => 120) or die "Couldn't connect to server";
}
elsif( $port == '587' && $host =~ m/office365/ ){
	print "Office365 mode\n" if $debug;
	#connexion office365
	$smtp = Net::SMTPS->new(Host =>$host,Port=> $port,Timeout => 120,doSSL => 'starttls',Debug=>0) or die "Couldn't connect to server";
}
else{
	print "Classic mode\n" if $debug;
	#connexion classique
	$smtp = Net::SMTP->new(Host =>$host,Hello=>$hello ,Port=> $port,Timeout => 120) or die "Couldn't connect to server";
}
if(defined $user){
        $smtp->auth($user ,$password) or die "Could not authenticate $! \n quit";
}

#récupération des informations sur l'erreur
$query = $db->prepare("SELECT device_name,device_ip_addr,device_description,device_description,error_name,error_type,to_char(happen_timestamp,'DD/MM/YYYY HH24:MI:SS tz') AS happen_timestamp,happen_message,
				service_name,service_parameter,esia_device_group.device_group_name as device_group_name,happen_acquitte,4 as device_ip_version
                          FROM esia_error_happen
                          JOIN esia_device ON happen_device_id = device_id
                          JOIN esia_errors ON happen_error_id= error_id
                          JOIN esia_services ON happen_service_id= service_id
                          LEFT JOIN esia_device_group_in ON group_in_device_id=device_id
                          LEFT JOIN esia_device_group ON device_group_id=group_in_device_group_id
                          WHERE happen_id=?");
$query->execute($opts{'e'});
my $data = $query->fetchrow_hashref;
if($data)
{
        if($data->{happen_acquitte} == 1)
        {
                $data->{error_type}="ok";
                $data->{happen_message}=": status changed to ok";
        }
	$subject = format_mail($subject,$data);
#	$subject = encode("MIME-Header",$subject);
	$message = format_mail($message,$data);
    $message = encode('UTF-8', $message);
    $subject = encode('UTF-8', $subject);
	print "to $to subject $subject\n" if $debug;

	$smtp->mail("$from");
	$smtp->to( split (/[,;]/,$to));
	$smtp->data();
	$smtp->datasend("To: $to\n");
	$smtp->datasend("From: $from\n");
	$smtp->datasend("Content-Type: text/html; charset=\"UTF-8\"\n");
	$smtp->datasend("X-Mailer: Perl Sendmail \n");
	$smtp->datasend("Subject: $subject\n");
	$smtp->datasend("\n");
	$smtp->datasend("$message\n<CR><LF>.<CR><LF>");
	$smtp->dataend();
	$smtp->quit();
}
else
{
	print "NO DATA FOUND => CHECK error_happen_id please\n" if $debug;
}
$query->finish;
$db->disconnect;
exit(0);
